<?php

namespace App\Controller\Config;

use App\Entity\Machine;
use App\Entity\Material;
use App\Form\Config\MachineEditForm;
use App\Form\Config\MaterialEditForm;
use App\Misc\FlashType;
use App\Repository\MachineRepository;
use App\Repository\MaterialRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;

/**
 * Controller for the configuration pages of material
 */
#[Route('/Config/Material')]
class MaterialController extends AbstractController {
    private readonly EntityManagerInterface $entityManager;
    /**
     * @var MaterialRepository The material repository
     */
    private readonly MaterialRepository $materialRepository;

    /**
     * Initialization
     *
     * @param EntityManagerInterface $entityManager      The entity manager
     * @param MaterialRepository     $materialRepository The material repository
     */
    public function __construct (EntityManagerInterface $entityManager, MaterialRepository $materialRepository) {
        $this->entityManager = $entityManager;
        $this->materialRepository = $materialRepository;
    }

    /**
     * List of materials
     *
     * @return Response The response
     */
    #[Route('/', name: 'config_material_list', alias: 'config_material')]
    public function list (): Response {
        return $this->render(
            'Config/Material/List.html.twig',
            [
                'materials' => $this->materialRepository->findAll(),
            ]
        );
    }

    /**
     * Edit/Create a material
     *
     * @param Request      $request  The request
     * @param Material|null $material The material to edit
     *
     * @return Response The response
     */
    #[Route('/Create', name: 'config_material_create')]
    #[Route('/Edit-{id}', name: 'config_material_edit')]
    public function edit (Request $request, ?Material $material = null): Response {
        $material ??= new Material();

        $form = $this->createForm(MaterialEditForm::class, $material);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $this->entityManager->persist($material);
            $this->entityManager->flush();

            $this->addFlash(FlashType::SUCCESS, 'Le matériau a bien été enregistré.');
            return $this->redirectToRoute('config_material_list');
        }

        return $this->render('Config/Material/Edit.html.twig', [
            'material' => $material,
            'form'    => $form->createView(),
        ]);
    }

    /**
     * Delete a material
     *
     * @param Material $material The material to delete
     *
     * @return Response The response
     */
    #[Route('/Delete-{id}', name: 'config_material_delete')]
    public function delete (Material $material): Response {
        $this->entityManager->remove($material);
        $this->entityManager->flush();

        $this->addFlash(FlashType::SUCCESS, 'Le matériau a bien été supprimé.');
        return $this->redirectToRoute('config_material_list');
    }
}
